@@ -88,7 +88,11 @@ async def show_async(
                 network_space_human_readable = blockchain_state["space"] / 1024 ** 4
                 if network_space_human_readable >= 1024:
                     network_space_human_readable = network_space_human_readable / 1024
-                    print(f"{network_space_human_readable:.3f} PiB")
+                    if network_space_human_readable >= 1024:
+                        network_space_human_readable = network_space_human_readable / 1024
+                        print(f"{network_space_human_readable:.3f} EiB")
+                    else:
+                        print(f"{network_space_human_readable:.3f} PiB")
                 else:
                     print(f"{network_space_human_readable:.3f} TiB")
                 print(f"Current difficulty: {difficulty}")
@@ -261,7 +265,7 @@ async def show_async(
                 print("Block with header hash", block_header_hash_by_height, "not found")
 
     except Exception as e:
-        if isinstance(e, aiohttp.client_exceptions.ClientConnectorError):
+        if isinstance(e, aiohttp.ClientConnectorError):
             print(f"Connection error. Check if full node rpc is running at {rpc_port}")
             print("This is normal if full node is still starting up")
         else:
