@@ -14,11 +14,10 @@ from chia.util.ws_message import WsRpcMessage, create_payload_dict
 
 
 class DaemonProxy:
-    def __init__(self, uri, ssl_context):
+    def __init__(self, uri: str, ssl_context: Optional[ssl.SSLContext]):
         self._uri = uri
         self._request_dict: Dict[bytes32, asyncio.Event] = {}
         self.response_dict: Dict[bytes32, Any] = {}
-        self.websocket = None
         self.ssl_context = ssl_context
 
     def format_request(self, command: str, data: Dict[str, Any]) -> WsRpcMessage:
