@@ -89,6 +89,24 @@ class FullNodeRpcClient(RpcClient):
             for coin in ((await self.fetch("get_coin_records_by_puzzle_hash", d))["coin_records"])
         ]
 
+    async def get_coin_records_by_puzzle_hashes(
+        self,
+        puzzle_hashes: List[bytes32],
+        include_spent_coins: bool = True,
+        start_height: Optional[int] = None,
+        end_height: Optional[int] = None,
+    ) -> List:
+        puzzle_hashes_hex = [ph.hex() for ph in puzzle_hashes]
+        d = {"puzzle_hashes": puzzle_hashes_hex, "include_spent_coins": include_spent_coins}
+        if start_height is not None:
+            d["start_height"] = start_height
+        if end_height is not None:
+            d["end_height"] = end_height
+        return [
+            CoinRecord.from_json_dict(coin)
+            for coin in ((await self.fetch("get_coin_records_by_puzzle_hashes", d))["coin_records"])
+        ]
+
     async def get_additions_and_removals(self, header_hash: bytes32) -> Tuple[List[CoinRecord], List[CoinRecord]]:
         try:
             response = await self.fetch("get_additions_and_removals", {"header_hash": header_hash.hex()})
