@@ -139,13 +139,15 @@ class ChiaServer:
         self.app_shut_down_task: Optional[asyncio.Task] = None
         self.received_message_callback: Optional[Callable] = None
         self.api_tasks: Dict[bytes32, asyncio.Task] = {}
+        self.execute_tasks: Set[bytes32] = set()
+
         self.tasks_from_peer: Dict[bytes32, Set[bytes32]] = {}
         self.banned_peers: Dict[str, float] = {}
         self.invalid_protocol_ban_seconds = 10
         self.api_exception_ban_seconds = 10
 
     def my_id(self) -> bytes32:
-        """ If node has public cert use that one for id, if not use private."""
+        """If node has public cert use that one for id, if not use private."""
         if self.p2p_crt_path is not None:
             pem_cert = x509.load_pem_x509_certificate(self.p2p_crt_path.read_bytes(), default_backend())
         else:
@@ -465,6 +467,8 @@ class ChiaServer:
 
         task_ids = self.tasks_from_peer[peer_id]
         for task_id in task_ids:
+            if task_id in self.execute_tasks:
+                continue
             task = self.api_tasks[task_id]
             task.cancel()
 
@@ -501,6 +505,12 @@ class ChiaServer:
                         if self.api.api_ready is False:
                             return None
 
+                    timeout: Optional[int] = 600
+                    if hasattr(f, "execute_task"):
+                        # Don't timeout on methods with execute_task decorator, these need to run fully
+                        self.execute_tasks.add(task_id)
+                        timeout = None
+
                     if hasattr(f, "peer_required"):
                         coroutine = f(full_message.data, connection)
                     else:
@@ -518,7 +528,7 @@ class ChiaServer:
                             raise e
                         return None
 
-                    response: Optional[Message] = await asyncio.wait_for(wrapped_coroutine(), timeout=600)
+                    response: Optional[Message] = await asyncio.wait_for(wrapped_coroutine(), timeout=timeout)
                     connection.log.debug(
                         f"Time taken to process {message_type} from {connection.peer_node_id} is "
                         f"{time.time() - start_time} seconds"
@@ -542,6 +552,8 @@ class ChiaServer:
                         self.api_tasks.pop(task_id)
                     if task_id in self.tasks_from_peer[connection.peer_node_id]:
                         self.tasks_from_peer[connection.peer_node_id].remove(task_id)
+                    if task_id in self.execute_tasks:
+                        self.execute_tasks.remove(task_id)
 
             task_id = token_bytes()
             api_task = asyncio.create_task(api_call(payload_inc, connection_inc, task_id))
