@@ -91,7 +91,12 @@ class WSChiaConnection:
         self.request_results: Dict[bytes32, Message] = {}
         self.closed = False
         self.connection_type: Optional[NodeType] = None
-        self.request_nonce: uint16 = uint16(0)
+        if is_outbound:
+            self.request_nonce: uint16 = uint16(0)
+        else:
+            # Different nonce to reduce chances of overlap. Each peer will increment the nonce by one for each
+            # request. The receiving peer (not is_outbound), will use 2^15 to 2^16 - 1
+            self.request_nonce = uint16(2 ** 15)
 
         # This means that even if the other peer's boundaries for each minute are not aligned, we will not
         # disconnect. Also it allows a little flexibility.
@@ -272,7 +277,7 @@ class WSChiaConnection:
         return invoke
 
     async def create_request(self, message_no_id: Message, timeout: int) -> Optional[Message]:
-        """ Sends a message and waits for a response. """
+        """Sends a message and waits for a response."""
         if self.closed:
             return None
 
@@ -280,8 +285,14 @@ class WSChiaConnection:
         event = asyncio.Event()
 
         # The request nonce is an integer between 0 and 2**16 - 1, which is used to match requests to responses
+        # If is_outbound, 0 <= nonce < 2^15, else  2^15 <= nonce < 2^16
         request_id = self.request_nonce
-        self.request_nonce = uint16(self.request_nonce + 1) if self.request_nonce != (2 ** 16 - 1) else uint16(0)
+        if self.is_outbound:
+            self.request_nonce = uint16(self.request_nonce + 1) if self.request_nonce != (2 ** 15 - 1) else uint16(0)
+        else:
+            self.request_nonce = (
+                uint16(self.request_nonce + 1) if self.request_nonce != (2 ** 16 - 1) else uint16(2 ** 15)
+            )
 
         message = Message(message_no_id.type, request_id, message_no_id.data)
 
