@@ -40,37 +40,19 @@ class SpendBundle(Streamable):
         return items
 
     def removals(self) -> List[Coin]:
-        """ This should be used only by wallet"""
+        """This should be used only by wallet"""
         return [_.coin for _ in self.coin_solutions]
 
     def fees(self) -> int:
-        """ Unsafe to use for fees validation!!! """
+        """Unsafe to use for fees validation!!!"""
         amount_in = sum(_.amount for _ in self.removals())
         amount_out = sum(_.amount for _ in self.additions())
 
         return amount_in - amount_out
 
-    def removal_names(self) -> List[bytes32]:
-        return [_.coin.name() for _ in self.coin_solutions]
-
-    def addition_names(self) -> List[bytes32]:
-        return [_.name() for _ in self.additions()]
-
     def name(self) -> bytes32:
         return self.get_hash()
 
-    def not_ephemeral_spends(self):
-        all_removals = self.removals()
-        all_additions = self.additions()
-        result: List[Coin] = []
-
-        for rem in all_removals:
-            if rem in all_additions:
-                continue
-            result.append(rem)
-
-        return result
-
     def not_ephemeral_additions(self):
         all_removals = self.removals()
         all_additions = self.additions()
