@@ -49,6 +49,9 @@ class DIDWallet:
         num_of_backup_ids_needed: uint64 = None,
         name: str = None,
     ):
+        """
+        This must be called under the wallet state manager lock
+        """
         self = DIDWallet()
         self.base_puzzle_program = None
         self.base_inner_puzzle_hash = None
@@ -80,9 +83,9 @@ class DIDWallet:
         if spend_bundle is None:
             raise ValueError("failed to generate ID for wallet")
         await self.wallet_state_manager.add_new_wallet(self, self.wallet_info.id)
-
+        assert self.did_info.origin_coin is not None
         did_puzzle_hash = did_wallet_puzzles.create_fullpuz(
-            self.did_info.current_inner, self.did_info.my_did
+            self.did_info.current_inner, self.did_info.origin_coin.puzzle_hash
         ).get_tree_hash()
 
         did_record = TransactionRecord(
@@ -238,58 +241,57 @@ class DIDWallet:
         return uint64(result)
 
     async def select_coins(self, amount, exclude: List[Coin] = None) -> Optional[Set[Coin]]:
-        """ Returns a set of coins that can be used for generating a new transaction. """
-        async with self.wallet_state_manager.lock:
-            if exclude is None:
-                exclude = []
+        """Returns a set of coins that can be used for generating a new transaction."""
+        if exclude is None:
+            exclude = []
 
-            spendable_amount = await self.get_spendable_balance()
-            if amount > spendable_amount:
-                self.log.warning(f"Can't select {amount}, from spendable {spendable_amount} for wallet id {self.id()}")
-                return None
+        spendable_amount = await self.get_spendable_balance()
+        if amount > spendable_amount:
+            self.log.warning(f"Can't select {amount}, from spendable {spendable_amount} for wallet id {self.id()}")
+            return None
 
-            self.log.info(f"About to select coins for amount {amount}")
-            unspent: List[WalletCoinRecord] = list(
-                await self.wallet_state_manager.get_spendable_coins_for_wallet(self.wallet_info.id)
+        self.log.info(f"About to select coins for amount {amount}")
+        unspent: List[WalletCoinRecord] = list(
+            await self.wallet_state_manager.get_spendable_coins_for_wallet(self.wallet_info.id)
+        )
+        sum_value = 0
+        used_coins: Set = set()
+
+        # Use older coins first
+        unspent.sort(key=lambda r: r.confirmed_block_height)
+
+        # Try to use coins from the store, if there isn't enough of "unused"
+        # coins use change coins that are not confirmed yet
+        unconfirmed_removals: Dict[bytes32, Coin] = await self.wallet_state_manager.unconfirmed_removals_for_wallet(
+            self.wallet_info.id
+        )
+        for coinrecord in unspent:
+            if sum_value >= amount and len(used_coins) > 0:
+                break
+            if coinrecord.coin.name() in unconfirmed_removals:
+                continue
+            if coinrecord.coin in exclude:
+                continue
+            sum_value += coinrecord.coin.amount
+            used_coins.add(coinrecord.coin)
+
+        # This happens when we couldn't use one of the coins because it's already used
+        # but unconfirmed, and we are waiting for the change. (unconfirmed_additions)
+        if sum_value < amount:
+            raise ValueError(
+                "Can't make this transaction at the moment. Waiting for the change from the previous transaction."
             )
-            sum_value = 0
-            used_coins: Set = set()
-
-            # Use older coins first
-            unspent.sort(key=lambda r: r.confirmed_block_height)
-
-            # Try to use coins from the store, if there isn't enough of "unused"
-            # coins use change coins that are not confirmed yet
-            unconfirmed_removals: Dict[bytes32, Coin] = await self.wallet_state_manager.unconfirmed_removals_for_wallet(
-                self.wallet_info.id
-            )
-            for coinrecord in unspent:
-                if sum_value >= amount and len(used_coins) > 0:
-                    break
-                if coinrecord.coin.name() in unconfirmed_removals:
-                    continue
-                if coinrecord.coin in exclude:
-                    continue
-                sum_value += coinrecord.coin.amount
-                used_coins.add(coinrecord.coin)
-
-            # This happens when we couldn't use one of the coins because it's already used
-            # but unconfirmed, and we are waiting for the change. (unconfirmed_additions)
-            if sum_value < amount:
-                raise ValueError(
-                    "Can't make this transaction at the moment. Waiting for the change from the previous transaction."
-                )
 
         self.log.info(f"Successfully selected coins: {used_coins}")
         return used_coins
 
     # This will be used in the recovery case where we don't have the parent info already
     async def coin_added(self, coin: Coin, header_hash: bytes32, removals: List[Coin], height: int):
-        """ Notification from wallet state manager that wallet has been received. """
+        """Notification from wallet state manager that wallet has been received."""
         self.log.info("DID wallet has been notified that coin was added")
         inner_puzzle = await self.inner_puzzle_for_did_puzzle(coin.puzzle_hash)
         new_info = DIDInfo(
-            self.did_info.my_did,
+            self.did_info.origin_coin,
             self.did_info.backup_ids,
             self.did_info.num_of_backup_ids_needed,
             self.did_info.parent_info,
@@ -310,9 +312,12 @@ class DIDWallet:
 
     def create_backup(self, filename: str):
         assert self.did_info.current_inner is not None
+        assert self.did_info.origin_coin is not None
         try:
             f = open(filename, "w")
-            output_str = f"{self.get_my_DID()}:"
+            output_str = f"{self.did_info.origin_coin.parent_coin_info}:"
+            output_str += f"{self.did_info.origin_coin.puzzle_hash}:"
+            output_str += f"{self.did_info.origin_coin.amount}:"
             for did in self.did_info.backup_ids:
                 output_str = output_str + did.hex() + ","
             output_str = output_str[:-1]
@@ -330,16 +335,16 @@ class DIDWallet:
             f = open(filename, "r")
             details = f.readline().split(":")
             f.close()
-            genesis_id = bytes.fromhex(details[0])
+            origin = Coin(bytes.fromhex(details[0]), bytes.fromhex(details[1]), uint64(int(details[2])))
             backup_ids = []
-            for d in details[1].split(","):
+            for d in details[3].split(","):
                 backup_ids.append(bytes.fromhex(d))
-            num_of_backup_ids_needed = uint64(int(details[3]))
+            num_of_backup_ids_needed = uint64(int(details[5]))
             if num_of_backup_ids_needed > len(backup_ids):
                 raise Exception
-            innerpuz = Program.from_bytes(bytes.fromhex(details[2]))
+            innerpuz = Program.from_bytes(bytes.fromhex(details[4]))
             did_info = DIDInfo(
-                genesis_id,
+                origin,
                 backup_ids,
                 num_of_backup_ids_needed,
                 self.did_info.parent_info,
@@ -350,7 +355,8 @@ class DIDWallet:
             )
             await self.save_info(did_info, False)
             await self.wallet_state_manager.update_wallet_puzzle_hashes(self.wallet_info.id)
-            full_puz = did_wallet_puzzles.create_fullpuz(innerpuz, genesis_id)
+
+            full_puz = did_wallet_puzzles.create_fullpuz(innerpuz, origin.puzzle_hash)
             full_puzzle_hash = full_puz.get_tree_hash()
             (
                 sub_height,
@@ -395,7 +401,7 @@ class DIDWallet:
                         if coin.name() in all_parents:
                             continue
                         did_info = DIDInfo(
-                            genesis_id,
+                            origin,
                             backup_ids,
                             num_of_backup_ids_needed,
                             self.did_info.parent_info,
@@ -414,8 +420,10 @@ class DIDWallet:
         innerpuz = did_wallet_puzzles.create_innerpuz(
             pubkey, self.did_info.backup_ids, self.did_info.num_of_backup_ids_needed
         )
-        did = self.did_info.my_did
-        return did_wallet_puzzles.create_fullpuz(innerpuz, did)
+        if self.did_info.origin_coin is not None:
+            return did_wallet_puzzles.create_fullpuz(innerpuz, self.did_info.origin_coin.puzzle_hash)
+        else:
+            return did_wallet_puzzles.create_fullpuz(innerpuz, 0x00)
 
     async def get_new_puzzle(self) -> Program:
         return self.puzzle_for_pk(
@@ -423,13 +431,15 @@ class DIDWallet:
         )
 
     def get_my_DID(self) -> str:
-        core = self.did_info.my_did
+        assert self.did_info.origin_coin is not None
+        core = self.did_info.origin_coin.puzzle_hash
         assert core is not None
         return core.hex()
 
     # This is used to cash out, or update the id_list
     async def create_spend(self, puzhash: bytes32):
         assert self.did_info.current_inner is not None
+        assert self.did_info.origin_coin is not None
         coins = await self.select_coins(1)
         assert coins is not None
         coin = coins.pop()
@@ -440,13 +450,13 @@ class DIDWallet:
 
         full_puzzle: Program = did_wallet_puzzles.create_fullpuz(
             innerpuz,
-            self.did_info.my_did,
+            self.did_info.origin_coin.puzzle_hash,
         )
         parent_info = await self.get_parent_for_coin(coin)
         assert parent_info is not None
-
         fullsol = Program.to(
             [
+                [self.did_info.origin_coin.parent_coin_info, self.did_info.origin_coin.amount],
                 [
                     parent_info.parent_name,
                     parent_info.inner_puzzle_hash,
@@ -458,7 +468,11 @@ class DIDWallet:
         )
         list_of_solutions = [CoinSolution(coin, full_puzzle, fullsol)]
         # sign for AGG_SIG_ME
-        message = puzhash + coin.name() + self.wallet_state_manager.constants.AGG_SIG_ME_ADDITIONAL_DATA
+        message = (
+            Program.to([coin.amount, puzhash]).get_tree_hash()
+            + coin.name()
+            + self.wallet_state_manager.constants.AGG_SIG_ME_ADDITIONAL_DATA
+        )
         pubkey = did_wallet_puzzles.get_pubkey_from_innerpuz(innerpuz)
         index = await self.wallet_state_manager.puzzle_store.index_for_pubkey(pubkey)
         private = master_sk_to_wallet_sk(self.wallet_state_manager.private_key, index)
@@ -494,6 +508,7 @@ class DIDWallet:
         self, recovering_coin_name: bytes32, newpuz: bytes32, pubkey: G1Element, filename=None
     ) -> SpendBundle:
         assert self.did_info.current_inner is not None
+        assert self.did_info.origin_coin is not None
         coins = await self.select_coins(1)
         assert coins is not None and coins != set()
         coin = coins.pop()
@@ -505,13 +520,14 @@ class DIDWallet:
         innerpuz: Program = self.did_info.current_inner
         full_puzzle: Program = did_wallet_puzzles.create_fullpuz(
             innerpuz,
-            self.did_info.my_did,
+            self.did_info.origin_coin.puzzle_hash,
         )
         parent_info = await self.get_parent_for_coin(coin)
         assert parent_info is not None
 
         fullsol = Program.to(
             [
+                [self.did_info.origin_coin.parent_coin_info, self.did_info.origin_coin.amount],
                 [
                     parent_info.parent_name,
                     parent_info.inner_puzzle_hash,
@@ -526,7 +542,8 @@ class DIDWallet:
 
         message_spend_bundle = SpendBundle([message_spend], AugSchemeMPL.aggregate([]))
         # sign for AGG_SIG_ME
-        message = innermessage + coin.name() + self.wallet_state_manager.constants.AGG_SIG_ME_ADDITIONAL_DATA
+        to_sign = Program.to([coin.puzzle_hash, coin.amount, innermessage]).get_tree_hash()
+        message = to_sign + coin.name() + self.wallet_state_manager.constants.AGG_SIG_ME_ADDITIONAL_DATA
         pubkey = did_wallet_puzzles.get_pubkey_from_innerpuz(innerpuz)
         index = await self.wallet_state_manager.puzzle_store.index_for_pubkey(pubkey)
         private = master_sk_to_wallet_sk(self.wallet_state_manager.private_key, index)
@@ -622,6 +639,7 @@ class DIDWallet:
         pubkey: G1Element,
         spend_bundle: SpendBundle,
     ) -> SpendBundle:
+        assert self.did_info.origin_coin is not None
         # innerpuz solution is (mode amount new_puz identity my_puz parent_innerpuzhash_amounts_for_recovery_ids)
         innersol = Program.to(
             [
@@ -640,12 +658,13 @@ class DIDWallet:
         innerpuz = self.did_info.current_inner
         full_puzzle: Program = did_wallet_puzzles.create_fullpuz(
             innerpuz,
-            self.did_info.my_did,
+            self.did_info.origin_coin.puzzle_hash,
         )
         parent_info = await self.get_parent_for_coin(coin)
         assert parent_info is not None
         fullsol = Program.to(
             [
+                [self.did_info.origin_coin.parent_coin_info, self.did_info.origin_coin.amount],
                 [
                     parent_info.parent_name,
                     parent_info.inner_puzzle_hash,
@@ -735,23 +754,25 @@ class DIDWallet:
         return parent_info
 
     async def generate_new_decentralised_id(self, amount: uint64) -> Optional[SpendBundle]:
+        """
+        This must be called under the wallet state manager lock
+        """
 
         coins = await self.standard_wallet.select_coins(amount)
         if coins is None:
             return None
 
         origin = coins.copy().pop()
-        origin_id = origin.name()
 
         did_inner: Program = await self.get_new_innerpuz()
         did_inner_hash = did_inner.get_tree_hash()
-        did_puz = did_wallet_puzzles.create_fullpuz(did_inner, origin_id)
+        did_puz = did_wallet_puzzles.create_fullpuz(did_inner, origin.puzzle_hash)
         did_puzzle_hash = did_puz.get_tree_hash()
 
         tx_record: Optional[TransactionRecord] = await self.standard_wallet.generate_signed_transaction(
-            amount, did_puzzle_hash, uint64(0), origin_id, coins
+            amount, did_puzzle_hash, uint64(0), origin.name(), coins
         )
-        eve_coin = Coin(origin_id, did_puzzle_hash, amount)
+        eve_coin = Coin(origin.name(), did_puzzle_hash, amount)
         future_parent = CCParent(
             eve_coin.parent_coin_info,
             did_inner_hash,
@@ -770,7 +791,7 @@ class DIDWallet:
 
         # Only want to save this information if the transaction is valid
         did_info: DIDInfo = DIDInfo(
-            origin_id,
+            origin,
             self.did_info.backup_ids,
             self.did_info.num_of_backup_ids_needed,
             self.did_info.parent_info,
@@ -780,18 +801,30 @@ class DIDWallet:
             None,
         )
         await self.save_info(did_info, False)
-        eve_spend = await self.generate_eve_spend(eve_coin, did_puz, origin_id, did_inner)
+        eve_spend = await self.generate_eve_spend(eve_coin, did_puz, did_inner)
         full_spend = SpendBundle.aggregate([tx_record.spend_bundle, eve_spend])
         return full_spend
 
-    async def generate_eve_spend(self, coin: Coin, full_puzzle: Program, origin_id: bytes, innerpuz: Program):
+    async def generate_eve_spend(self, coin: Coin, full_puzzle: Program, innerpuz: Program):
+        assert self.did_info.origin_coin is not None
         # innerpuz solution is (mode amount message my_id my_puzhash parent_innerpuzhash_amounts_for_recovery_ids)
         innersol = Program.to([0, coin.amount, coin.puzzle_hash, coin.name(), coin.puzzle_hash, []])
         # full solution is (parent_info my_amount innersolution)
-        fullsol = Program.to([coin.parent_coin_info, coin.amount, innersol])
+        fullsol = Program.to(
+            [
+                [self.did_info.origin_coin.parent_coin_info, self.did_info.origin_coin.amount],
+                coin.parent_coin_info,
+                coin.amount,
+                innersol,
+            ]
+        )
         list_of_solutions = [CoinSolution(coin, full_puzzle, fullsol)]
         # sign for AGG_SIG_ME
-        message = coin.puzzle_hash + coin.name() + self.wallet_state_manager.constants.AGG_SIG_ME_ADDITIONAL_DATA
+        message = (
+            Program.to([coin.amount, coin.puzzle_hash]).get_tree_hash()
+            + coin.name()
+            + self.wallet_state_manager.constants.AGG_SIG_ME_ADDITIONAL_DATA
+        )
         pubkey = did_wallet_puzzles.get_pubkey_from_innerpuz(innerpuz)
         index = await self.wallet_state_manager.puzzle_store.index_for_pubkey(pubkey)
         private = master_sk_to_wallet_sk(self.wallet_state_manager.private_key, index)
@@ -820,7 +853,7 @@ class DIDWallet:
         current_list = self.did_info.parent_info.copy()
         current_list.append((name, parent))
         did_info: DIDInfo = DIDInfo(
-            self.did_info.my_did,
+            self.did_info.origin_coin,
             self.did_info.backup_ids,
             self.did_info.num_of_backup_ids_needed,
             current_list,
@@ -835,7 +868,7 @@ class DIDWallet:
         if num_of_backup_ids_needed > len(recover_list):
             return False
         did_info: DIDInfo = DIDInfo(
-            self.did_info.my_did,
+            self.did_info.origin_coin,
             recover_list,
             num_of_backup_ids_needed,
             self.did_info.parent_info,
