@@ -64,8 +64,8 @@ def get_discrepancies_for_spend_bundle(
     try:
         cc_discrepancies: Dict[str, int] = dict()
         for coinsol in trade_offer.coin_solutions:
-            puzzle = coinsol.puzzle_reveal
-            solution = coinsol.solution
+            puzzle: Program = Program.from_bytes(bytes(coinsol.puzzle_reveal))
+            solution: Program = Program.from_bytes(bytes(coinsol.solution))
             # work out the deficits between coin amount and expected output for each
             r = cc_utils.uncurry_cc(puzzle)
             if r:
