@@ -3,13 +3,11 @@ from dataclasses import dataclass
 from chia.types.blockchain_format.coin import Coin
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.util.ints import uint32
-from chia.util.streamable import Streamable, streamable
 from chia.wallet.util.wallet_types import WalletType
 
 
 @dataclass(frozen=True)
-@streamable
-class WalletCoinRecord(Streamable):
+class WalletCoinRecord:
     """
     These are values that correspond to a CoinName that are used
     in keeping track of the unspent database.
