@@ -1,14 +1,17 @@
 from collections import defaultdict
 from dataclasses import dataclass, replace
-from typing import Dict, Iterator, List
+from typing import Dict, Iterator, Set
 
-from chia.consensus.blockchain_check_conditions import blockchain_check_conditions_dict
-from chia.types.announcement import Announcement
+from chia.full_node.mempool_check_conditions import mempool_check_conditions_dict
 from chia.types.blockchain_format.coin import Coin
 from chia.types.blockchain_format.sized_bytes import bytes32
 from chia.types.coin_record import CoinRecord
 from chia.types.spend_bundle import SpendBundle
-from chia.util.condition_tools import conditions_dict_for_solution, created_announcements_for_conditions_dict
+from chia.util.condition_tools import (
+    conditions_dict_for_solution,
+    coin_announcement_names_for_conditions_dict,
+    puzzle_announcement_names_for_conditions_dict,
+)
 from chia.util.ints import uint32, uint64
 
 
@@ -37,34 +40,47 @@ class CoinStore:
         self,
         spend_bundle: SpendBundle,
         now: CoinTimestamp,
+        max_cost: int,
     ) -> int:
         # this should use blockchain consensus code
 
-        announcements: List[Announcement] = []
+        coin_announcements: Set[bytes32] = set()
+        puzzle_announcements: Set[bytes32] = set()
+
         conditions_dicts = []
         for coin_solution in spend_bundle.coin_solutions:
             err, conditions_dict, cost = conditions_dict_for_solution(
-                coin_solution.puzzle_reveal, coin_solution.solution
+                coin_solution.puzzle_reveal, coin_solution.solution, max_cost
             )
             if conditions_dict is None:
                 raise BadSpendBundleError(f"clvm validation failure {err}")
             conditions_dicts.append(conditions_dict)
-            announcements.extend(created_announcements_for_conditions_dict(conditions_dict, coin_solution.coin.name()))
+            coin_announcements.update(
+                coin_announcement_names_for_conditions_dict(conditions_dict, coin_solution.coin.name())
+            )
+            puzzle_announcements.update(
+                puzzle_announcement_names_for_conditions_dict(conditions_dict, coin_solution.coin.puzzle_hash)
+            )
 
         for coin_solution, conditions_dict in zip(spend_bundle.coin_solutions, conditions_dicts):
             prev_transaction_block_height = now.height
             timestamp = now.seconds
             coin_record = self._db[coin_solution.coin.name()]
-            err = blockchain_check_conditions_dict(
-                coin_record, announcements, conditions_dict, uint32(prev_transaction_block_height), uint64(timestamp)
+            err = mempool_check_conditions_dict(
+                coin_record,
+                coin_announcements,
+                puzzle_announcements,
+                conditions_dict,
+                uint32(prev_transaction_block_height),
+                uint64(timestamp),
             )
         if err is not None:
             raise BadSpendBundleError(f"condition validation failure {err}")
 
         return 0
 
-    def update_coin_store_for_spend_bundle(self, spend_bundle: SpendBundle, now: CoinTimestamp):
-        err = self.validate_spend_bundle(spend_bundle, now)
+    def update_coin_store_for_spend_bundle(self, spend_bundle: SpendBundle, now: CoinTimestamp, max_cost: int):
+        err = self.validate_spend_bundle(spend_bundle, now, max_cost)
         if err != 0:
             raise BadSpendBundleError(f"validation failure {err}")
         for spent_coin in spend_bundle.removals():
