@@ -6,12 +6,13 @@ from typing import List
 import aiosqlite
 import pytest
 
-from src.consensus.blockchain import Blockchain
-from src.consensus.constants import ConsensusConstants
-from src.full_node.block_store import BlockStore
-from src.full_node.coin_store import CoinStore
-from src.types.full_block import FullBlock
-from src.util.path import mkdir
+from chia.consensus.blockchain import Blockchain
+from chia.consensus.constants import ConsensusConstants
+from chia.full_node.block_store import BlockStore
+from chia.full_node.coin_store import CoinStore
+from chia.types.full_block import FullBlock
+from chia.util.db_wrapper import DBWrapper
+from chia.util.path import mkdir
 from tests.setup_nodes import bt, test_constants
 
 
@@ -20,8 +21,9 @@ async def create_blockchain(constants: ConsensusConstants):
     if db_path.exists():
         db_path.unlink()
     connection = await aiosqlite.connect(db_path)
-    coin_store = await CoinStore.create(connection)
-    store = await BlockStore.create(connection)
+    wrapper = DBWrapper(connection)
+    coin_store = await CoinStore.create(wrapper)
+    store = await BlockStore.create(wrapper)
     bc1 = await Blockchain.create(coin_store, store, constants)
     assert bc1.get_peak() is None
     return bc1, connection, db_path
